//+------------------------------------------------------------------+
//|                                    10Points3_Dynamic_Stop_v2.mq4 |
//|              Copyright  2006, Alejandro Galindo and Matt Tabone |
//|                                                                  |
//|                   This EA was developed from the following EA... |
//|      This newer version has been re-organized to make reading it |
//|       simpler.  Other additions include the mods listed below... |
//|                                                New order spacing |
//|                              New indicators for better direction |
//|                                                                  |
//|                                                   10points 3.mq4 |
//|                              Copyright  2005, Alejandro Galindo |
//|                                              http://elCactus.com |
//+------------------------------------------------------------------+

#property copyright "Copyright  2006, Alejandro Galindo and Matt Tabone"
#property link      "http://elCactus.com"

//---- input parameters
extern double     LotSize              = 0.1;               //Size of initial order
extern int        LotPrecision         = 1;                 //Used for NormalizeDouble to determine number of decimal places on lot size
extern double     TakeProfit           = 38;                //Profit goal for the last order placed
extern int        OrderSlippage        = 5;                 //Number of Pips +/- an order price can be off when being placed
extern int        MaxOrders            = 10;                //Maximum amount of orders to open
extern int        Pips                 = 15;                //Pip distance from the previous order if market goes against us
extern double     LotsIncreaseBy       = 1.6;               //New orders will be the previous size times this amount
extern bool       MoneyManagement      = false;             //Should orders be opened based on account percentage
extern int        RiskPercent          = 5;                 //Percentage of account used to place first order
extern int        AccountType          = 0;                 //0=Micro/Mini, 1=Standard
extern int        Magic                = 20061113;          //Used to track which orders where created by this EA

//---- internal parameters
bool              Synchronized         = false;             //Gets set to true after the first run thru when EA is started
double            ActualLotSize        = 0;                 //Gets set accordingly depending on MoneyManagement settings
int               OrderDirection       = 0;                 //0=none, 1=Long, 2=Short
double            ActualSLPoints       = 0;                 //Amount of points before all orders get stopped out
double            ActualSL             = 0;                 //Stop Loss for new order
double            ActualTP             = 0;                 //Take Profit for new order
int               ActualOrders         = 0;                 //Keeps track of which order we're on
double            OrderPL              = 0;                 //Gets set to current Profit/Loss for all open orders
int               ConfirmedOrders      = 0;                 //Count of actual open orders
int               cnt                  = 0;                 //Temporary counter variable
double            UpTrendVal           = 0;
double            DnTrendVal           = 0;
double            TrendVal             = 0;
int               TrendType            = 0;
string            TrendText            = "";
int               RSX_Period           = 17;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+

int init() { return(0); }

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+

int deinit() { return(0); }

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+

int start() {

   //Check any open orders and possibly synchronize
   CheckOpenOrders();

   //Have we got any open orders yet?
   if (ActualOrders == 0) { OpenInitialOrder(); }
   //Clean up open orders, close orders, place a new order
   else { OrderMaintenance(); }

   //Print status on chart
   ChartComment();

   return(0);

}

//+------------------------------------------------------------------+
//| extra functions in the order they are used/called                |
//+------------------------------------------------------------------+

void CheckOpenOrders() {

   ConfirmedOrders = 0;
   OrderPL = 0;
   //Get a count of the open orders, Profit/Loss total, possible synchronize
   //by looping thru all orders 
   for (cnt = 0; cnt < OrdersTotal(); cnt++) {
      //Select the current order
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      //Does the order match our symbol and magic number?
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic) {
         //If EA just started or was restarted with possible open orders
         if (Synchronized == false) {
            //Will end up being the last order's size for this symbol
            ActualLotSize = OrderLots();
            //Reset OrderDirection
            if (OrderType() == OP_BUY) {
               OrderDirection = 1;
            }
            else if (OrderType() == OP_SELL) {
               OrderDirection = 2;
            }
            //Reset ActualSLPoints, ActualSL, ActualTP, ActualOrders
            ActualSLPoints = MaxOrders * Pips;
            ActualSL = OrderStopLoss();
            ActualTP = OrderTakeProfit();
            ActualOrders++;
         }
         //Add to the total Profit/Loss
         OrderPL = OrderPL + OrderProfit() + OrderSwap();
         //Add to confirmed orders
         ConfirmedOrders++;
      }
   }

   Synchronized = true;

}

void OpenInitialOrder() {

   //Check/set money management
   CheckMoneyManagement();

   //Check to see which direction to go
   CheckForDirection();

   //Figure out maximum Stop Loss
   ActualSLPoints = MaxOrders * Pips;

   //Are we trying to go long?
   if (OrderDirection == 1) {
      //Set Stop Loss and Take Profit and open a Long order
      ActualSL = Ask - (ActualSLPoints * Point);
      ActualTP = Ask + (TakeProfit * Point);
      ActualOrders = 1;
      OrderSend(Symbol(), OP_BUY, ActualLotSize, Ask, OrderSlippage, ActualSL, ActualTP, "Order" + ActualOrders, Magic, 0, Blue);
   }
   //Are we trying to go short?
   else if (OrderDirection == 2) {
      //Set Stop Loss and Take Profit and open a Short order
      ActualSL = Bid + (ActualSLPoints * Point);
      ActualTP = Bid - (TakeProfit * Point);
      ActualOrders = 1;
      OrderSend(Symbol(), OP_SELL, ActualLotSize, Bid, OrderSlippage, ActualSL, ActualTP, "Order" + ActualOrders, Magic, 0, Red);
   }

}

void CheckMoneyManagement() {

   if (MoneyManagement == true) {
      //Set lot size for a Micro/Mini account?
      if (AccountType == 0) {
         ActualLotSize = MathCeil(AccountBalance() * RiskPercent / 10000) / 10;
      }
      //Set lot size for a Standard account?
      else if (AccountType == 1) {
         ActualLotSize = MathCeil(AccountBalance() * RiskPercent / 10000);
      }
   }
   //Set lot size to what end user has indicated in settings
   else { ActualLotSize = LotSize; }

}

void CheckForDirection() {

   UpTrendVal = iCustom(Symbol(), PERIOD_H4, "Turbo_JVEL",7,-100,0,0);
   DnTrendVal = iCustom(Symbol(), PERIOD_H4, "Turbo_JVEL",7,-100,1,0);
   TrendVal = (UpTrendVal + DnTrendVal);

   //Strong Uptrend
   if(TrendVal >= 0.1) {
      TrendType = 1;
      TrendText = "Strong Uptrend";
   }
   //Weak Uptrend/Ranging
   if(TrendVal > 0 && TrendVal < 0.1) {
      TrendType = 2;
      TrendText = "Weak Uptrend/Ranging";
   }

   //Strong Downtrend
   if(TrendVal <= -0.1) {
      TrendType = 3;
      TrendText = "Strong Downtrend";
   }
   //Weak Downtrend/Ranging
   if(TrendVal > -0.1 && TrendVal < 0) {
      TrendType = 4;
      TrendText = "Weak Downtrend/Ranging";
   }

   double rsxcurr = iCustom(Symbol(),Period(),"Turbo_JRSX",RSX_Period,0,0);
   double rsxprev = iCustom(Symbol(),Period(),"Turbo_JRSX",RSX_Period,0,1);
   if ((rsxcurr > rsxprev) && (TrendType == 1 || TrendType == 2)) { OrderDirection = 1; }
   if ((rsxcurr < rsxprev) && (TrendType == 3 || TrendType == 4)) { OrderDirection = 2; }

}

void OrderMaintenance() {

   //Are we trying to go long?
   if (OrderDirection == 1) {
      //If the Stop Loss or Take Profit has been achieved, or our order count doesn't
      //match actual open orders then we close them all
      if (Bid <= ActualSL || Bid >= ActualTP || ConfirmedOrders < ActualOrders) {
         OrderNeedsClosing();
      }
      //If we're under our max orders allowed and current price has fallen by our designated
      //amount from the last order then we need to open another order
      else if (ActualOrders < MaxOrders && Ask <= ActualTP - ((TakeProfit + Pips) * Point)) {
         OrderNeedsOpening();
      }
   }
   //Are we trying to go short?
   else if (OrderDirection == 2) {
      //If the Stop Loss or Take Profit has been achieved, or our order count doesn't
      //match actual open orders then we close them all
      if (Ask >= ActualSL || Ask <= ActualTP || ConfirmedOrders < ActualOrders) {
         OrderNeedsClosing();
      }
      //If we're under our max orders allowed and current price has risen by our designated
      //amount from the last order then we need to open another order
      else if (ActualOrders < MaxOrders && Bid >= ActualTP + ((TakeProfit + Pips) * Point)) {
         OrderNeedsOpening();
      }
   }

}

void OrderNeedsClosing() {

   //Loop thru all open orders
   for (cnt = OrdersTotal(); cnt >= 0; cnt--) {
      //Select the current order
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      //Does the order match our symbol and magic number?
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic) {
         //If long, close the order
         if (OrderDirection == 1) {
            OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), OrderSlippage, Blue);
         }
         //If short, close the order
         else if (OrderDirection == 2) {
            OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), OrderSlippage, Red);
         }
      }
   }
   ActualOrders = 0;

}

void OrderNeedsOpening() {

   //Are we trying to go long?
   if (OrderDirection == 1) {
      //Increase Lot Size, Set Take Profit and open a Long order
      ActualLotSize = NormalizeDouble(ActualLotSize * LotsIncreaseBy, LotPrecision);
      ActualTP = Ask + (TakeProfit * Point);
      ActualOrders++;
      OrderSend(Symbol(), OP_BUY, ActualLotSize, Ask, OrderSlippage, ActualSL, ActualTP, "Order" + ActualOrders, Magic, 0, Blue);
   }
   //Are we trying to go short?
   else if (OrderDirection == 2) {
      //Increase Lot Size, Set Take Profit and open a Short order
      ActualLotSize = NormalizeDouble(ActualLotSize * LotsIncreaseBy, LotPrecision);
      ActualTP = Bid - (TakeProfit * Point);
      ActualOrders++;
      OrderSend(Symbol(), OP_SELL, ActualLotSize, Bid, OrderSlippage, ActualSL, ActualTP, "Order" + ActualOrders, Magic, 0, Red);
   }

}

void ChartComment() {

   string sComment   = "";
   string sLine      = "-------------------------------------------------";
   string sNewLine   = "\n";

   sComment = sLine + sNewLine;
   sComment = sComment + "Synchronized: " + Synchronized + " / ActualLotSize: " + DoubleToStr(ActualLotSize, 2) + " / OrderDirection: " + OrderDirection + sNewLine;
   sComment = sComment + "ActualSLPoints: " + DoubleToStr(ActualSLPoints, 2) + " / ActualSL: " + DoubleToStr(ActualSL, 4) + " / ActualTP: " + DoubleToStr(ActualTP, 4) + sNewLine;
   sComment = sComment + "ConfirmedOrders: " + ConfirmedOrders + " / ActualOrders: " + ActualOrders + " / OrderPL: $" + DoubleToStr(OrderPL, 2) + sNewLine;
   sComment = sComment + "Current Bid: " + DoubleToStr(Bid, 4) + " / Current Ask: " + DoubleToStr(Ask, 4) + sNewLine;
   sComment = sComment + "Total orders open on account: " + cnt + sNewLine;
   sComment = sComment + "UpTrendVal: " + DoubleToStr(UpTrendVal, 5) + " / DnTrendVal: " + DoubleToStr(DnTrendVal, 5) + " / TrendVal: " + DoubleToStr(TrendVal, 5) + sNewLine;
   sComment = sComment + "TrendType: " + TrendType + " / TrendText: " + TrendText + sNewLine;
   sComment = sComment + "RSX_Period: " + RSX_Period + sNewLine;
   sComment = sComment + sLine + sNewLine;

   Comment(sComment);

}



